package com.asc.server.objects;

/**
 * @created 27-Jan-2007 9:30:48 PM
 * @author Travis Kennedy
 * @version 1.0
 */
import com.core.server.objects.*;
import javax.servlet.http.*;
import com.core.server.common.*;

public class Investment {


    protected int asset_class_id;
    protected int sector_id;
    protected int investment_id;
    protected int type_id;

    protected String companyName;
    protected String tickerSymbol;
    protected String cusip;
    protected String marketCap;
    protected float EPS;
    protected float PE;
    protected long volume; 

    protected int positionsHeld;

    protected DateTime createdDate;

    public Investment(){
        asset_class_id = 0;
        sector_id = 0;
        type_id = 0;
        investment_id = 0;

        companyName= new String("");
        marketCap= new String("");
        tickerSymbol= new String("");
        cusip= new String("");
        positionsHeld= 0;
        EPS = 0.0f;
        PE = 0.0f;
        volume = 0;
   }

   public Investment(int portfolioId){
        asset_class_id = portfolioId;
        sector_id = 0;
        type_id = 0;
        investment_id = 0;
        positionsHeld= 0;
        companyName= new String("");
        marketCap= new String("");
        tickerSymbol= new String("");
        cusip= new String("");
        EPS = 0.0f;
        PE = 0.0f;
        volume = 0;
    }


    public Investment(Investment pInvestment) {
        asset_class_id = pInvestment.asset_class_id;
        sector_id = pInvestment.sector_id;
        type_id = pInvestment.type_id;
        investment_id = pInvestment.investment_id;
        positionsHeld= pInvestment.positionsHeld;
        marketCap= new String(pInvestment.marketCap);
        companyName= new String(pInvestment.companyName);
        tickerSymbol= new String(pInvestment.tickerSymbol);
        cusip= new String(pInvestment.cusip);
        EPS = 0.0f;
        PE = 0.0f;
        volume = 0;


    }

    public Investment(HttpServletRequest request) throws Exception {
        asset_class_id = 0;

        try {

            String sector_id = null;
            if (request.getParameter("sector_id") != null) {
                sector_id = new String(request.getParameter("sector_id"));
                setSectorId(new Integer(sector_id).intValue());
            }

            String type_id = null;
            if (request.getParameter("type_id") != null) {
                type_id = new String(request.getParameter("type_id"));
                setTypeId(new Integer(type_id).intValue());
            }

            String investment_id = null;
            if (request.getParameter("investment_id") != null) {
                investment_id = new String(request.getParameter("investment_id"));
                setInvestmentId(new Integer(investment_id).intValue());
            }

            String asset_class_id = null;
            if (request.getParameter("asset_class_id") != null) {
                asset_class_id = new String(request.getParameter("asset_class_id"));
                setInvestmentId(new Integer(asset_class_id).intValue());
            }


            String company_name = null;
            if (request.getParameter("company_name") != null) {
                company_name = new String(request.getParameter("company_name"));
                setCompanyName(company_name);
            }

            String ticker_symbol = null;
            if (request.getParameter("ticker_symbol") != null) {
                ticker_symbol = new String(request.getParameter("ticker_symbol"));
                setTickerSymbol(ticker_symbol);
            }

            String cusip = null;
            if (request.getParameter("cusip") != null) {
                cusip = new String(request.getParameter("cusip"));
                setCusip(cusip);
            }

            String market_cap = null;
            if (request.getParameter("market_cap") != null) {
                market_cap = new String(request.getParameter("market_cap"));
                setMarketCap(market_cap);
            }
		} catch (Exception e) {
		            throw e;
		}
    }



    /**
     * unique identifier of the Investment table
     */
    public int getAssetClassId(){
        return asset_class_id;
    }

    public String getAssetClassIdString() {
        return Integer.toString(asset_class_id);
    }

    public int getSectorId(){
        return sector_id;
    }

    public String getSectorIdString() {
        return Integer.toString(sector_id);
    }
    public int getTypeId(){
        return type_id;
    }

    public String getTypeIdString() {
        return Integer.toString(type_id);
    }

    public int getInvestmentId(){
        return investment_id;
    }

    public String getInvestmentIdString() {
        return Integer.toString(investment_id);
    }


    public String getCompanyName(){
        return companyName;
    }

    public String getTickerSymbol(){
        return tickerSymbol;
    }


    public int getPositionsHeld(){
        return positionsHeld;
    }


    public String getPositionsHeldString() {
        return Integer.toString(positionsHeld);
    }

    public String getCusip(){
        return cusip;
    }

    public String getMarketCap(){
        return marketCap;
    }

    
    public float getEPS() {
		return EPS;
	}

	public void setEPS(float EPS) {
		this.EPS = EPS;
	}

	public float getPE() {
		return PE;
	}

	public void setPE(float PE) {
		this.PE = PE;
	}

	public long getVolume() {
		return volume;
	}

	public void setVolume(long volume) {
		this.volume = volume;
	}

	public DateTime getCreatedDate(){
        return createdDate;
    }


    public String getCreatedDateDBString() {
        if (this.createdDate != null && this.createdDate.isValid()) {
            return this.createdDate.getYear()+"-"+
                    this.createdDate.getMonth()+"-"+
                    this.createdDate.getDay();
        } else {
            return new String("");
        }

    }

    public String getCreatedDateString() {
        if (this.createdDate != null && this.createdDate.isValid()) {
            return this.createdDate.getDate("/");
        } else {
            return new String("");
        }

    }


    /**
     * unique identifier of the Investment table
     *
     * @param newVal    newVal
     */
    public void setAssetClassId(int newVal){
        asset_class_id = newVal;
    }

    /**
     * unique identifier of the Investment table
     *
     * @param newVal    newVal
     */
    public void setTypeId(int newVal){
        type_id = newVal;
    }

    /**
     * unique identifier of the Investment table
     *
     * @param newVal    newVal
     */
    public void setInvestmentId(int newVal){
        investment_id = newVal;
    }



    public void setPositionsHeld(int newVal){
        positionsHeld = newVal;
    }




    /**
     * unique identifier of the Investment table
     *
     * @param newVal    newVal
     */
    public void setSectorId(int newVal){
        sector_id = newVal;
    }

    /**
     *
     * @param newVal    newVal
     */
    public void setCusip(String newVal){
        this.cusip = newVal;
    }

    public void setMarketCap(String newVal){
        this.marketCap = newVal;
    }

    public void setTickerSymbol(String newVal){
        this.tickerSymbol = newVal;
    }

    public void setCompanyName(String newVal){
        this.companyName = newVal;
    }
    public void setCreatedDate(DateTime newVal){
        this.createdDate = newVal;
    }



    public void setCreatedDate(String pDate) {
        if (pDate != null) {
            AOGStringTokenizer st = new AOGStringTokenizer(pDate, "-");

            // Parse out the year
            String endYear = new String(st.nextElement());

            // Parse out the day
            String endMonth = new String(st.nextElement());

            // Parse out the month
            String endDay = new String(st.nextElement());

            AOGStringTokenizer dayToken = new AOGStringTokenizer(endDay, " ");

            String day = dayToken.nextElement();

            DateTime pBirthDate = new DateTime(endMonth + "/" + day + "/" + endYear);
            setCreatedDate(pBirthDate);
        }

    }
    public String toString() {
        String returnString = "Investment Id: " + getInvestmentIdString() + " | " +
        "Asset Class Id: " + getAssetClassIdString() + " | " +
        "Sector Id: " + getSectorIdString() + " | " +
        "Type Id: " + getTypeIdString() + " | " +
        "Company: "  + getCompanyName() + " | " +
        "Ticket Symbol: "  + getTickerSymbol() + " | " +
        "Cusip: "  + getCusip() + " | " +
        "Market Cap: " + getMarketCap() + " | " +
        "Volume: " + getVolume() + " | " +
        "EPS: " + getEPS() + " | " + 
        "PE: " + getPE() + " | ";

        return (returnString);
    }

    /**
     *
     * @exception Throwable Throwable
     */
    public void finalize()
    throws Throwable{

    }
}