package com.asc.servlets;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.asc.applet.ui.objects.CompanyInfo;
import com.asc.applet.ui.objects.CubeInfo;
import com.asc.applet.ui.objects.InnerCube;
import com.asc.server.database.AssetClassDB;
import com.asc.server.database.InvestmentHoldingDB;
import com.asc.server.database.PortfolioAxesDefinitionDB;
import com.asc.server.database.SectorDB;
import com.asc.server.objects.AssetClass;
import com.asc.server.objects.Axis;
import com.asc.server.objects.InvestmentSummary;
import com.asc.server.objects.PortfolioAxesDefinition;
import com.asc.server.objects.Sector;
import com.asc.server.objects.User;
import com.core.server.loggers.AOGLogger;

public class CubeInfoServlet extends HttpServlet {
		
	protected AOGLogger logger = null;
	private static final String className = "CubeInfoServlet";
	private HashMap availableColorMap = new HashMap();
	private HashMap currentColorMap = new HashMap();
	
	String denomination = null;
	
	public void init()throws ServletException {
		
		this.logger = (AOGLogger) this.getServletContext().getAttribute("logger");
		AOGLogger.setLevel(2);
		
		setAvailableColorMap();
		
	}
	
	
	public void doGet(HttpServletRequest request, HttpServletResponse response)  throws ServletException, IOException {

        doPost(request,response);

    }
	
	
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
				
		String location = className + "::: doPost() :";
		
		ObjectOutputStream out = new ObjectOutputStream(response.getOutputStream());
		Map responseMap = new HashMap();
		
		// Get the UserId and PortfolioId		
		String portfolioId = request.getParameter("portfolioId");
		User userProfile = (User) request.getSession(false).getAttribute("userProfile");
        int userId = userProfile.getUserId();
        
        
        String message = "portfolioId => " + portfolioId + "; userProfile => " + userProfile;
        this.logger.debugMessage(message, location);
		
        // Get ParentSectorId(available only for drilldown screen)
        int parentSectorId = 0;
        int assetClassId = 0;
        if( (request.getParameter("parentSectorId") != null)  && 
        			(request.getParameter("assetClassId") != null) ) {
        	 parentSectorId = Integer.parseInt(request.getParameter("parentSectorId"));
        	 assetClassId = Integer.parseInt(request.getParameter("assetClassId"));        	 
        }
        
        
        message = "parentSectorId => " + parentSectorId + "; assetClassId => " + assetClassId;
        this.logger.debugMessage(message, location);
        
        
        //=====================================================================
        
        // Determine X Axis, Y Axis and Z Axis
        PortfolioAxesDefinition portfolioAxesDefinition = null; 
        try {	
        	portfolioAxesDefinition = 
        	  new PortfolioAxesDefinitionDB(this.getServletContext()).getPortfolioAxesDefinition(portfolioId);        	
        } catch (Exception e) {
            this.logger.debugMessage("Exception in PortfolioAxesDefinitionDB retrieve", location);
        } 
       
        
        Axis xAxis = portfolioAxesDefinition.getXAxis();
        Axis yAxis = portfolioAxesDefinition.getYAxis();
        Axis zAxis = portfolioAxesDefinition.getZAxis();
        
        
        this.logger.debugMessage("xAxis = " + xAxis.getLabelType(), location);
        this.logger.debugMessage("yAxis = " + yAxis.getLabelType(), location);
        this.logger.debugMessage("zAxis = " + zAxis.getLabelType(), location);
        
        //======================================================================
        				
		// Initialize the Asset Class Axis Labels from db
        ArrayList assetClassList = null;
         
        try {
        	assetClassList = (ArrayList) new AssetClassDB(this.getServletContext()).getAssetClassList();
        } catch (Exception e) {        	
            this.logger.debugMessage("Exception in Asset List retrieval", location);
        }
        
        int numOfElements = assetClassList.size();
        String[] assetClassAxisLabels = new String[numOfElements];
        
        int j = 0;
        for(Iterator<AssetClass> iter=assetClassList.iterator(); iter.hasNext();) {
        	assetClassAxisLabels[j] = iter.next().getAssetClassName();
        	j++;
        }
        
        //======================================================================
                
        // Initialize the Sector Axis Labels from db
        ArrayList sectorList = null;
        
        try {
        	
        	// if parent sector id is not zeros, then it is a drill down screen
        	// if parent sector id is zeros, then it is the main screen
        	if(parentSectorId != 0 && assetClassId != 0) {
        		sectorList = (ArrayList) new InvestmentHoldingDB(this.getServletContext()).
        			getSubSectorsForInvestment(Integer.parseInt(portfolioId), assetClassId, parentSectorId);
        		
        	} else {        		
        		sectorList = (ArrayList) new SectorDB(this.getServletContext()).getSectorList();
        	}
        } catch (Exception e) {        	
            this.logger.debugMessage("Exception in Sector List retrieval", location);
        }
        
        numOfElements = sectorList.size();
        String[] sectorAxisLabels = new String[numOfElements];
        
        this.logger.debugMessage("sector list = ");
        j = 0;
        
        if(parentSectorId != 0 && assetClassId != 0) {
        	for(Iterator<String> iter=sectorList.iterator(); iter.hasNext();) {
        		sectorAxisLabels[j] = iter.next();
        		this.logger.debugMessage(sectorAxisLabels[j] + ", ");
        		j++;        	
        	}
        } else {
        	for(Iterator<Sector> iter=sectorList.iterator(); iter.hasNext();) {
        		sectorAxisLabels[j] = iter.next().getSectorName();
        		this.logger.debugMessage(sectorAxisLabels[j] + ", ");
        		j++;        	
        	}
        	
        }
        this.logger.debugMessage(" ");
        

        
        //======================================================================
        
        // Generate the Frequency Axis Labels using maxscale, minscale and interval
        // values from db
        
        String[] frequencyAxisLabels = null;
        String frequencyAxis = null;
        
        // Figure out which axis is Market Capitalization if any specified for this protfolio Id
        Axis marketCapAxis = null;
        if(xAxis.getLabelType().equals("Market Cap")) {
        	marketCapAxis = xAxis;
        	frequencyAxis = "xAxis";
        } else if(yAxis.getLabelType().equals("Market Cap")) {
        	marketCapAxis = yAxis;
        	frequencyAxis = "yAxis";
        } else if(zAxis.getLabelType().equals("Market Cap")) {
        	marketCapAxis = zAxis;
        	frequencyAxis = "zAxis";
        }
                
        // Only if Market Capitalization has been specified continue with
        // Initializing the Axis Label Arrays - Market Capitalization     
        // Currently only the minimum scale value is taken into consideration
        // while deciding whether the denomination is in "B"illions or "M"illions.
        
        if( marketCapAxis != null) {        
        	
        	long scaleMin = marketCapAxis.getScaleMin();
        	long scaleMax = marketCapAxis.getScaleMax();
        	long scaleInterval = marketCapAxis.getInterval();
        	
        	String scaleMinStr = Long.toString(scaleMin);
        	denomination = "";
        	
        	if(scaleMinStr.length() > 9) {
        		denomination = "B";
        	} else if(scaleMinStr.length() > 6) {
        		denomination = "M";
        	}
        	        	
        	
        	this.logger.debugMessage("Market Cap scaleMin/scaleMax/scaleInterval/denomination = " + scaleMin + "/" 
        					+ scaleMax + "/" + scaleInterval + "/" + denomination);
        	this.logger.debugMessage("Market Cap = ");
        	ArrayList marketCapList = new ArrayList();
        	for(long i = scaleMin; i <= scaleMax; i=i+scaleInterval) {
        		
        		if(denomination.equals("M")){
        		   marketCapList.add(Double.toString((i/1000000.0d)) + denomination);
        		   System.out.print(Long.toString((i/1000000)) + denomination + ", ");
        		} else if(denomination.equals("B")) {
        		   marketCapList.add(Double.toString((i/1000000000.0d)) + denomination);
         		   System.out.print(Long.toString((i/1000000000)) + denomination + ", ");	
        		}
        	}
        	
        	
        	
        	j = 0;
        	frequencyAxisLabels = new String[marketCapList.size()];
        	for(Iterator<String> iter=marketCapList.iterator(); iter.hasNext();) {
        		frequencyAxisLabels[j] = iter.next();
        		j++;
        	}
        	
        }
        
        
        // Figure out which axis is Volume if any specified for this protfolio Id
        Axis volumeAxis = null;
        if(xAxis.getLabelType().equals("Volume")) {
        	volumeAxis = xAxis;
        	frequencyAxis = "xAxis";
        } else if(yAxis.getLabelType().equals("Volume")) {
        	volumeAxis = yAxis;
        	frequencyAxis = "yAxis";
        } else if(zAxis.getLabelType().equals("Volume")) {
        	volumeAxis = zAxis;
        	frequencyAxis = "zAxis";
        }
                
        // Only if Volume has been specified continue with
        // Initializing the Axis Label Arrays - Volume     
        
        if( volumeAxis != null) {        
        	
        	long scaleMin = volumeAxis.getScaleMin();
        	long scaleMax = volumeAxis.getScaleMax();
        	long scaleInterval = volumeAxis.getInterval();
        	
        	ArrayList volumeList = new ArrayList();
        	for(long i = scaleMin; i <= scaleMax; i=i+scaleInterval) {
        		
        		volumeList.add(Double.toString(i));
        	}
        	
        	j = 0;
        	frequencyAxisLabels = new String[volumeList.size()];
        	for(Iterator<String> iter=volumeList.iterator(); iter.hasNext();) {
        		frequencyAxisLabels[j] = iter.next();
        		j++;
        	}
        	
        }
        
        
        
        // Figure out which axis is P/E if any specified for this protfolio Id
        Axis PEAxis = null;
        if(xAxis.getLabelType().equals("P/E")) {
        	PEAxis = xAxis;
        	frequencyAxis = "xAxis";
        } else if(yAxis.getLabelType().equals("P/E")) {
        	PEAxis = yAxis;
        	frequencyAxis = "yAxis";
        } else if(zAxis.getLabelType().equals("P/E")) {
        	PEAxis = zAxis;
        	frequencyAxis = "zAxis";
        }
                
        // Only if P/E has been specified continue with
        // Initializing the Axis Label Arrays - P/E     
        
        if( PEAxis != null) {        
        	
        	long scaleMin = PEAxis.getScaleMin();
        	long scaleMax = PEAxis.getScaleMax();
        	long scaleInterval = PEAxis.getInterval();
        	
        	ArrayList PEList = new ArrayList();
        	for(long i = scaleMin; i <= scaleMax; i=i+scaleInterval) {
        		
        		PEList.add(Double.toString(i));
        	}
        	
        	j = 0;
        	frequencyAxisLabels = new String[PEList.size()];
        	for(Iterator<String> iter=PEList.iterator(); iter.hasNext();) {
        		frequencyAxisLabels[j] = iter.next();
        		j++;
        	}
        	
        }
        
        
        
        
        // Figure out which axis is EPS if any specified for this protfolio Id
        Axis EPSAxis = null;
        if(xAxis.getLabelType().equals("EPS")) {
        	EPSAxis = xAxis;
        	frequencyAxis = "xAxis";
        } else if(yAxis.getLabelType().equals("EPS")) {
        	EPSAxis = yAxis;
        	frequencyAxis = "yAxis";
        } else if(zAxis.getLabelType().equals("EPS")) {
        	EPSAxis = zAxis;
        	frequencyAxis = "zAxis";
        }
                
        // Only if EPS has been specified continue with
        // Initializing the Axis Label Arrays - EPS     
        
        if( EPSAxis != null) {        
        	
        	long scaleMin = EPSAxis.getScaleMin();
        	long scaleMax = EPSAxis.getScaleMax();
        	long scaleInterval = EPSAxis.getInterval();
        	
        	ArrayList EPSList = new ArrayList();
        	for(long i = scaleMin; i <= scaleMax; i=i+scaleInterval) {
        		
        		EPSList.add(Double.toString(i));
        	}
        	
        	j = 0;
        	frequencyAxisLabels = new String[EPSList.size()];
        	for(Iterator<String> iter=EPSList.iterator(); iter.hasNext();) {
        		frequencyAxisLabels[j] = iter.next();
        		j++;
        	}
        	
        }
        
        
        
        
        
        
        // ======================================================================
        // Generate the inner Cubes
        		
        ArrayList investmentSummaryList = null;
        
        try {
        	
        	// if parent sector id is not zeros, then it is a drill down screen
        	// if parent sector id is zeros, then it is the main screen
        	if(parentSectorId != 0 && assetClassId != 0) {
        		
        		investmentSummaryList = (ArrayList) new InvestmentHoldingDB(this.getServletContext()).
				getInvestmentSummaryListForAssetClassAndParentSector(new Integer(portfolioId), assetClassId, 
						parentSectorId);
        		
        	} else {
        		
        		investmentSummaryList = (ArrayList) new InvestmentHoldingDB(this.getServletContext()).
				getInvestmentSummaryListBySector(new Integer(portfolioId));
        		
        	}
        } catch (Exception e) {        	
            this.logger.debugMessage("Exception in Investment SummaryList retrieval", location);
        }	
        
        
		InnerCube[] innerCubes = new InnerCube[investmentSummaryList.size()];
		InnerCube innerCube = null;
		CompanyInfo companyInfo = null;
		String[] subSectors = null;
		
		int i = 0;
		int availableColorMapIdx = 0;
		
		for(Iterator<InvestmentSummary> iter=investmentSummaryList.iterator(); iter.hasNext();) {
			
			InvestmentSummary investmentSummary = iter.next();
			AssetClass assetClass = null;
			Sector sector = null;
			
			logger.debugMessage("Setting innerCube Volume ::: " + investmentSummary.getVolume());
			logger.debugMessage("Setting innerCube EPS ::: " + investmentSummary.getEPS());
			logger.debugMessage("Setting innerCube PE ::: " + investmentSummary.getPE());
			
			try {
				assetClass = new AssetClassDB(this.getServletContext()).
						getAssetClass(Integer.toString(investmentSummary.getAssetClass_id()));
				sector = new SectorDB(this.getServletContext()).getSector(Integer.toString(investmentSummary.getParentSector_id()));
				
	        } catch (Exception e) {	        	
	            this.logger.debugMessage("Exception in Investment Summary retrieval", location);
	        }
	        
	        
			
			innerCube = new InnerCube();			
			
			// populate the axis lable types
			innerCube.setXAxisType(xAxis.getLabelType());
			innerCube.setYAxisType(yAxis.getLabelType());
			innerCube.setZAxisType(zAxis.getLabelType());
			
			// populate the X axis values
			if(xAxis.getLabelType().equals("Asset Class")) {
				innerCube.setXAxisValue(assetClass.getAssetClassName());
				
			} else if(xAxis.getLabelType().equals("Sector")) {
	        	innerCube.setXAxisValue(sector.getSectorName());
	        	
	        } else if(xAxis.getLabelType().equals("Market Cap")){	        	
	        	if(denomination.equals("M")) {
	        		long marketCap = new Long(investmentSummary.getMarketCap()).longValue();
	        		double marketCapM = marketCap/1000000;
	        		innerCube.setXAxisValue(marketCapM + "M");
				} else if(denomination.equals("B")) {
					long marketCap = new Long(investmentSummary.getMarketCap()).longValue();
	        		double marketCapB = marketCap/1000000000.0d;
	        		innerCube.setXAxisValue(marketCapB + "B");					
				} 	        	
	        	
	        } else if(xAxis.getLabelType().equals("Volume")) {
	        	innerCube.setXAxisValue(new Long(investmentSummary.getVolume()).toString());
	        	
	        } else if(xAxis.getLabelType().equals("EPS")) {
	        	innerCube.setXAxisValue(new Float(investmentSummary.getEPS()).toString());	        	
	        	
	        } else if(xAxis.getLabelType().equals("P/E")) {
	        	innerCube.setXAxisValue(new Float(investmentSummary.getPE()).toString());	        	
	        }
			
			
			// populate the Y axis values
			if(yAxis.getLabelType().equals("Asset Class")) {
				innerCube.setYAxisValue(assetClass.getAssetClassName());
				
			} else if(yAxis.getLabelType().equals("Sector")) {
	        	innerCube.setYAxisValue(sector.getSectorName());
	        	
	        } else if(yAxis.getLabelType().equals("Market Cap")){	        	
	        	if(denomination.equals("M")) {
	        		long marketCap = new Long(investmentSummary.getMarketCap()).longValue();
	        		double marketCapM = marketCap/1000000.0d;
	        		innerCube.setYAxisValue(marketCapM + "M");
				} else if(denomination.equals("B")) {
					long marketCap = new Long(investmentSummary.getMarketCap()).longValue();
					double marketCapB = marketCap/1000000000;
	        		innerCube.setYAxisValue(marketCapB + "B");					
				} 	        	
	        } else if(yAxis.getLabelType().equals("Volume")) {	        	
	        	innerCube.setYAxisValue(new Long(investmentSummary.getVolume()).toString());
	        		        	
	        } else if(yAxis.getLabelType().equals("EPS")) {
	        	innerCube.setYAxisValue(new Float(investmentSummary.getEPS()).toString());
	        	
	        } else if(yAxis.getLabelType().equals("P/E")) {
	        	innerCube.setYAxisValue(new Float(investmentSummary.getPE()).toString());
	        }
			
			
			// populate the Z axis values
			if(zAxis.getLabelType().equals("Asset Class")) {
				innerCube.setZAxisValue(assetClass.getAssetClassName());
				
			} else if(zAxis.getLabelType().equals("Sector")) {
	        	innerCube.setZAxisValue(sector.getSectorName());
	        	
	        } else if(zAxis.getLabelType().equals("Market Cap")){	        	
	        	if(denomination.equals("M")) {
	        		long marketCap = new Long(investmentSummary.getMarketCap()).longValue();
	        		double marketCapM = marketCap/1000000.0d;
	        		innerCube.setZAxisValue(marketCapM + "M");
				} else if(denomination.equals("B")) {
					long marketCap = new Long(investmentSummary.getMarketCap()).longValue();
					double marketCapB = marketCap/1000000000;
	        		innerCube.setZAxisValue(marketCapB + "B");					
				} 	        	
	        } else if(zAxis.getLabelType().equals("Volume")) {
	        	innerCube.setZAxisValue(new Long(investmentSummary.getVolume()).toString());
	        		        	
	        } else if(zAxis.getLabelType().equals("EPS")) {
	        	innerCube.setZAxisValue(new Float(investmentSummary.getEPS()).toString());
	        	
	        } else if(zAxis.getLabelType().equals("P/E")) {
	        	innerCube.setZAxisValue(new Float(investmentSummary.getPE()).toString());
	        	
	        }
			
			innerCube.setSector(sector.getSectorName());
			innerCube.setSectorId(sector.getSectorId());
			innerCube.setAssetClassId(investmentSummary.getAssetClass_id());
			
			
			// frequencyValue can either be Market Cap, Volume, 
			// P/E or EPS values			
			String frequencyValue = null;
			if(frequencyAxis.equals("xAxis")) {
				frequencyValue = innerCube.getXAxisValue();
				
			} else if(frequencyAxis.equals("yAxis")) {
				frequencyValue = innerCube.getYAxisValue();
			
			} else if(frequencyAxis.equals("zAxis")) {
				frequencyValue = innerCube.getZAxisValue();				
			}
			
			float[] color = null;
			if(currentColorMap.containsKey(frequencyValue)) {
				
				color = (float[]) currentColorMap.get(frequencyValue);
								
			} else {
			
				color = (float[]) availableColorMap.get(availableColorMapIdx);				
				currentColorMap.put(frequencyValue, color);
				availableColorMapIdx++;
				
				// If more than 50 different colors, reset idx back to 0.
				// We will not cross 50 as per current estimations
				if(availableColorMapIdx > 30) {
					availableColorMapIdx = 0;
				}
			}	
			
			innerCube.setColor(color);
			
			
			// find all subsectors for the investment's parent sector
			ArrayList subSectorsList = null;
			try {
	        	subSectorsList = (ArrayList) new InvestmentHoldingDB(this.getServletContext()).	        	
	        		getSubSectorsForInvestment(new Integer(portfolioId), assetClass.getAssetClassId(), 
	        				investmentSummary.getParentSector_id());
	        } catch (Exception e) {	        	
	            this.logger.debugMessage("Exception in subSectorsList retrieval", location);
	        }	
	        
	        subSectors = new String[subSectorsList.size()];
	        subSectorsList.toArray(subSectors);
			
			innerCube.setSubSectors(subSectors);
			
			companyInfo = new CompanyInfo();
			companyInfo.setName(investmentSummary.getCompanyName());
			companyInfo.setTicker(investmentSummary.getTickerSymbol());
			innerCube.setCompanyInfo(companyInfo);
			innerCubes[i] = innerCube;
			
			this.logger.debugMessage("innerCube xAxis / yAxis / zAxis = " + innerCube.getXAxisValue() + " / " +
					innerCube.getYAxisValue() + " / " + innerCube.getZAxisValue());
				
			i++;
		}
		
		
		// ======================================================================
        // Figure out the axis labels for the Outer Cube
                
        CubeInfo cubeInfo = new CubeInfo();
        
        // Populate X axis labels depending on the values from db
        cubeInfo.setXAxisTitle(xAxis.getLabelType());
        if(xAxis.getLabelType().equals("Asset Class")) {
        	cubeInfo.setXAxisLabels(assetClassAxisLabels);
        } else if(xAxis.getLabelType().equals("Sector")) {
        	cubeInfo.setXAxisLabels(sectorAxisLabels);
        } else {
        	cubeInfo.setXAxisLabels(frequencyAxisLabels);
        }
        
        
        // Populate Y axis labels depending on the values from db
        cubeInfo.setYAxisTitle(yAxis.getLabelType());
        if(yAxis.getLabelType().equals("Asset Class")) {
        	cubeInfo.setYAxisLabels(assetClassAxisLabels);
        } else if(yAxis.getLabelType().equals("Sector")) {
        	cubeInfo.setYAxisLabels(sectorAxisLabels);
        } else {
        	cubeInfo.setYAxisLabels(frequencyAxisLabels);
        }
        
        
        // Populate Z axis labels depending on the values from db
        cubeInfo.setZAxisTitle(zAxis.getLabelType());
        if(zAxis.getLabelType().equals("Asset Class")) {
        	cubeInfo.setZAxisLabels(assetClassAxisLabels);
        } else if(zAxis.getLabelType().equals("Sector")) {
        	cubeInfo.setZAxisLabels(sectorAxisLabels);
        } else {
        	cubeInfo.setZAxisLabels(frequencyAxisLabels);
        }
        
        
        // ======================================================================        
        // Populate the outer cube with inner cubes
		cubeInfo.setInnerCubes(innerCubes);
		
		
		
		// ======================================================================        
        // Prepare data to be sent back from the servlet to applet
		/*
		put watever objects you wants to return to applet.
		All objects must be serializable. */
		responseMap.put("Level1", cubeInfo);
		
		
		out.writeObject(responseMap);
		out.flush();
		out.close();
		response.setStatus(HttpServletResponse.SC_NO_CONTENT);
		
	}
	
	
	private void setAvailableColorMap() {
		
		availableColorMap.put(0, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(1, new float[] {0.9333f, 0.3058f, 0.5882f}); // pink
		availableColorMap.put(2, new float[] {1.0f, 0.0f, 0.3019f}); // pinkish red
		availableColorMap.put(3, new float[] {1.0f, 0.2f, 0.0f}); // orange
		availableColorMap.put(4, new float[] {0.7019f, 0.0f, 1.0f}); // lavender
		availableColorMap.put(5, new float[] {0.3019f, 1.0f, 0.0f}); // bright green
		availableColorMap.put(6, new float[] {0.0f, 0.8f, 1.0f}); // cyan
		availableColorMap.put(7, new float[] {0.2392f, 0.5450f, 1.0f}); // blue like jsp page
		availableColorMap.put(8, new float[] {0.0f, 0.4f, 0.6f}); // greyish blue
		availableColorMap.put(9, new float[] {0.0f, 0.6f, 0.1019f}); // green
		availableColorMap.put(10, new float[] {0.6f, 0.1019f, 0.0f}); // brick		
		availableColorMap.put(11, new float[] {0.5019f, 0.6f, 0.0f}); // dung
		availableColorMap.put(12, new float[] {0.6f, 0.4f, 0.0f}); // brown
		availableColorMap.put(13, new float[] {0.6f, 0.0f, 0.5019f}); // dark magenta
		availableColorMap.put(14, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(15, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(16, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(17, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(18, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(19, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(20, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(21, new float[] {0.0f, 1.0f, 0.0f}); // green		
		availableColorMap.put(22, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(23, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(24, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(25, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(26, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(27, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(28, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(29, new float[] {0.0f, 1.0f, 0.0f}); // green
		availableColorMap.put(30, new float[] {0.0f, 1.0f, 0.0f}); // green
		
		
	}

	

}
