package com.asc.servlets;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.asc.applet.ui.objects.CompanyInfo;
import com.asc.applet.ui.objects.CubeInfo;
import com.asc.applet.ui.objects.InnerCube;
import com.asc.server.database.AssetClassDB;
import com.asc.server.database.InvestmentHoldingDB;
import com.asc.server.database.PortfolioAxesDefinitionDB;
import com.asc.server.database.SectorDB;
import com.asc.server.objects.AssetClass;
import com.asc.server.objects.Axis;
import com.asc.server.objects.Investment;
import com.asc.server.objects.PortfolioAxesDefinition;
import com.asc.server.objects.Sector;
import com.asc.server.objects.User;
import com.core.server.loggers.AOGLogger;

public class TickerInfoServlet extends HttpServlet {
	
	protected AOGLogger logger = null;
	private static final String className = "TickerInfoServlet.";
	
	String denomination = null;
	
	
	public void init()throws ServletException {
	
	this.logger = (AOGLogger) this.getServletContext().getAttribute("logger");
	
	}
	
	
	public void doGet(HttpServletRequest request, HttpServletResponse response)  throws ServletException, IOException {

        doPost(request,response);

    }
	
	
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		ObjectOutputStream out = new ObjectOutputStream(response.getOutputStream());
		Map responseMap = new HashMap();
		
		// Get and load the search Ticker Symbol(s) which might also come as
		// comma delimited string, if user wanted to search more than one symbol
		// at a time
		String tickerSymbol = request.getParameter("tickerSymbol");
		String[] searchTickerArr = tickerSymbol.split(",");
		
		HashMap searchTickerMap = new HashMap();
		for(int i=0; i<searchTickerArr.length; i++) {
			searchTickerMap.put(i, searchTickerArr[i]);
		}
				
				
		// Get the UserId and PortfolioId		
		String portfolioId = request.getParameter("portfolioId");
		User userProfile = (User) request.getSession(false).getAttribute("userProfile");
        int userId = userProfile.getUserId();
		
        // Get ParentSectorId(available only for drilldown screen)
        int parentSectorId = 0;
        int assetClassId = 0;
        if( (request.getParameter("parentSectorId") != null)  && 
        			(request.getParameter("assetClassId") != null) ) {
        	 parentSectorId = Integer.parseInt(request.getParameter("parentSectorId"));
        	 assetClassId = Integer.parseInt(request.getParameter("assetClassId"));        	 
        }
        
        //=====================================================================
		
		
		
        // Determine X Axis, Y Axis and Z Axis
        PortfolioAxesDefinition portfolioAxesDefinition = null; 
        try {	
        	portfolioAxesDefinition = 
        	  new PortfolioAxesDefinitionDB(this.getServletContext()).getPortfolioAxesDefinition(portfolioId);        	
        } catch (Exception e) {
        	System.out.println("Exception in PortfolioAxesDefinitionDB retrieve");
        	String location = className + "doPost()";
            this.logger.debugMessage(location);
        } 
       
        
        Axis xAxis = portfolioAxesDefinition.getXAxis();
        Axis yAxis = portfolioAxesDefinition.getYAxis();
        Axis zAxis = portfolioAxesDefinition.getZAxis();
        
        System.out.println("xAxis = " + xAxis.getLabelType());
        System.out.println("yAxis = " + yAxis.getLabelType());
        System.out.println("zAxis = " + zAxis.getLabelType());
        
        //======================================================================
        				
		// Initialize the Asset Class Axis Labels from db
        ArrayList assetClassList = null;
         
        try {
        	assetClassList = (ArrayList) new AssetClassDB(this.getServletContext()).getAssetClassList();
        } catch (Exception e) {
        	String location = className + "doPost()";
            this.logger.debugMessage(location);
        }
        
        int numOfElements = assetClassList.size();
        String[] assetClassAxisLabels = new String[numOfElements];
        
        int j = 0;
        for(Iterator<AssetClass> iter=assetClassList.iterator(); iter.hasNext();) {
        	assetClassAxisLabels[j] = iter.next().getAssetClassName();
        	j++;
        }
        
        //======================================================================
                
        // Initialize the Sector Axis Labels from db
        ArrayList sectorList = null;
        
        try {
        	
        	// if parent sector id is not zeros, then it is a drill down screen
        	// if parent sector id is zeros, then it is the main screen
        	if(parentSectorId != 0 && assetClassId != 0) {
        		sectorList = (ArrayList) new InvestmentHoldingDB(this.getServletContext()).
        			getSubSectorsForInvestment(Integer.parseInt(portfolioId), assetClassId, parentSectorId);
        		
        	} else {        		
        		sectorList = (ArrayList) new SectorDB(this.getServletContext()).getSectorList();
        	}
        } catch (Exception e) {
        	String location = className + "doPost()";
            this.logger.debugMessage(location);
        }
        
        numOfElements = sectorList.size();
        String[] sectorAxisLabels = new String[numOfElements];
        
        System.out.print("sector list = ");
        j = 0;
        
        if(parentSectorId != 0 && assetClassId != 0) {
        	for(Iterator<String> iter=sectorList.iterator(); iter.hasNext();) {
        		sectorAxisLabels[j] = iter.next();
        		System.out.print(sectorAxisLabels[j] + ", ");
        		j++;        	
        	}
        } else {
        	for(Iterator<Sector> iter=sectorList.iterator(); iter.hasNext();) {
        		sectorAxisLabels[j] = iter.next().getSectorName();
        		System.out.print(sectorAxisLabels[j] + ", ");
        		j++;        	
        	}
        	
        }
        System.out.println(" ");
        

        
        //======================================================================
        
        // Generate the Frequency Axis Labels using maxscale, minscale and interval
        // values from db
        
        String[] frequencyAxisLabels = null;
        
        // Figure out which axis is Market Capitalization if any specified for this protfolio Id
        Axis marketCapAxis = null;
        if(xAxis.getLabelType().equals("Market Cap")) {
        	marketCapAxis = xAxis;
        } else if(yAxis.getLabelType().equals("Market Cap")) {
        	marketCapAxis = yAxis;
        } else if(zAxis.getLabelType().equals("Market Cap")) {
        	marketCapAxis = zAxis;
        }
                
        // Only if Market Capitalization has been specified continue with
        // Initializing the Axis Label Arrays - Market Capitalization     
        
        if( marketCapAxis != null) {        
        	
        	long scaleMin = marketCapAxis.getScaleMin();
        	long scaleMax = marketCapAxis.getScaleMax();
        	long scaleInterval = marketCapAxis.getInterval();
        	
        	String scaleMinStr = Long.toString(scaleMin);
        	denomination = "";
        	
        	if(scaleMinStr.length() > 9) {
        		denomination = "B";
        	} else if(scaleMinStr.length() > 6) {
        		denomination = "M";
        	}
        	        	
        	
        	logger.debugMessage("Market Cap scaleMin/scaleMax/scaleInterval = " + scaleMin + "/" + scaleMax + "/" + scaleInterval);
        	logger.debugMessage("Market Cap = ");
        	ArrayList marketCapList = new ArrayList();
        	for(long i = scaleMin; i <= scaleMax; i=i+scaleInterval) {
        		
        		if(denomination.equals("M")){
        		   marketCapList.add(Double.toString((i/1000000.0d)) + denomination);
        		   System.out.print(Long.toString((i/1000000)) + denomination + ", ");
        		} else if(denomination.equals("B")) {
        		   marketCapList.add(Double.toString((i/1000000000.0d)) + denomination);
         		   System.out.print(Long.toString((i/1000000000)) + denomination + ", ");	
        		}
        	}
        	
        	
        	j = 0;
        	frequencyAxisLabels = new String[marketCapList.size()];
        	for(Iterator<String> iter=marketCapList.iterator(); iter.hasNext();) {
        		frequencyAxisLabels[j] = iter.next();
        		j++;
        	}
        	
        }
        
        
        // Figure out which axis is Volume if any specified for this protfolio Id
        Axis volumeAxis = null;
        if(xAxis.getLabelType().equals("Volume")) {
        	volumeAxis = xAxis;
        } else if(yAxis.getLabelType().equals("Volume")) {
        	volumeAxis = yAxis;
        } else if(zAxis.getLabelType().equals("Volume")) {
        	volumeAxis = zAxis;
        }
                
        // Only if Volume has been specified continue with
        // Initializing the Axis Label Arrays - Volume     
        
        if( volumeAxis != null) {        
        	
        	long scaleMin = volumeAxis.getScaleMin();
        	long scaleMax = volumeAxis.getScaleMax();
        	long scaleInterval = volumeAxis.getInterval();
        	
        	ArrayList volumeList = new ArrayList();
        	for(long i = scaleMin; i <= scaleMax; i=i+scaleInterval) {
        		
        		volumeList.add(Double.toString(i));
        	}
        	
        	j = 0;
        	frequencyAxisLabels = new String[volumeList.size()];
        	for(Iterator<String> iter=volumeList.iterator(); iter.hasNext();) {
        		frequencyAxisLabels[j] = iter.next();
        		j++;
        	}
        	
        }
        
        
        
        // Figure out which axis is P/E if any specified for this protfolio Id
        Axis PEAxis = null;
        if(xAxis.getLabelType().equals("P/E")) {
        	PEAxis = xAxis;
        } else if(yAxis.getLabelType().equals("P/E")) {
        	PEAxis = yAxis;
        } else if(zAxis.getLabelType().equals("P/E")) {
        	PEAxis = zAxis;
        }
                
        // Only if P/E has been specified continue with
        // Initializing the Axis Label Arrays - P/E     
        
        if( PEAxis != null) {        
        	
        	long scaleMin = PEAxis.getScaleMin();
        	long scaleMax = PEAxis.getScaleMax();
        	long scaleInterval = PEAxis.getInterval();
        	
        	ArrayList PEList = new ArrayList();
        	for(long i = scaleMin; i <= scaleMax; i=i+scaleInterval) {
        		
        		PEList.add(Double.toString(i));
        	}
        	
        	j = 0;
        	frequencyAxisLabels = new String[PEList.size()];
        	for(Iterator<String> iter=PEList.iterator(); iter.hasNext();) {
        		frequencyAxisLabels[j] = iter.next();
        		j++;
        	}
        	
        }
        
        
        
       
//      Figure out which axis is EPS if any specified for this protfolio Id
        Axis EPSAxis = null;
        if(xAxis.getLabelType().equals("EPS")) {
        	EPSAxis = xAxis;
        } else if(yAxis.getLabelType().equals("EPS")) {
        	EPSAxis = yAxis;
        } else if(zAxis.getLabelType().equals("EPS")) {
        	EPSAxis = zAxis;
        }
                
        // Only if EPS has been specified continue with
        // Initializing the Axis Label Arrays - EPS     
        
        if( EPSAxis != null) {        
        	
        	long scaleMin = EPSAxis.getScaleMin();
        	long scaleMax = EPSAxis.getScaleMax();
        	long scaleInterval = EPSAxis.getInterval();
        	
        	ArrayList EPSList = new ArrayList();
        	for(long i = scaleMin; i <= scaleMax; i=i+scaleInterval) {
        		
        		EPSList.add(Double.toString(i));
        	}
        	
        	j = 0;
        	frequencyAxisLabels = new String[EPSList.size()];
        	for(Iterator<String> iter=EPSList.iterator(); iter.hasNext();) {
        		frequencyAxisLabels[j] = iter.next();
        		j++;
        	}
        	
        }
        
        
        
        // ======================================================================
        // Generate the inner Cubes
        		
        ArrayList investmentList = null;
        
        try {
        	
        	investmentList = (ArrayList) new InvestmentHoldingDB(this.getServletContext()).
			getInvestmentList(new Integer(portfolioId), "Ticker_Symbol");
        	
        } catch (Exception e) {
        	String location = className + "doPost()";
            this.logger.debugMessage(location);
        }	
        
        InnerCube[] innerCubes = null;
		ArrayList<InnerCube> innerCubesTmpArrList = new ArrayList<InnerCube>();
		InnerCube innerCube = null;
		CompanyInfo companyInfo = null;
		String[] subSectors = null;
		
		int i = 0;
		for(Iterator<Investment> iter=investmentList.iterator(); iter.hasNext();) {
			
			Investment investment = iter.next();
			AssetClass assetClass = null;
			Sector sector = null;
									
			// if ticker symbol is not the one searched for, then continue with 
			// the next until we find a match...
			if( !(searchTickerMap.containsValue(investment.getTickerSymbol())) ) {
								
				continue;
			}
						
			System.out.println("investment.getTickerSymbol()" + investment.getTickerSymbol());
			
			try {
				assetClass = new AssetClassDB(this.getServletContext()).
						getAssetClass(Integer.toString(investment.getAssetClassId()));
				sector = new SectorDB(this.getServletContext()).getSector(Integer.toString(investment.getSectorId()));
				
	        } catch (Exception e) {
	        	String location = className + "doPost()";
	            this.logger.debugMessage(location);
	        }
	        
	        
			
			innerCube = new InnerCube();			
			
			// populate the axis lable types
			innerCube.setXAxisType(xAxis.getLabelType());
			innerCube.setYAxisType(yAxis.getLabelType());
			innerCube.setZAxisType(zAxis.getLabelType());
			
			
			// populate the X axis values
			if(xAxis.getLabelType().equals("Asset Class")) {
				innerCube.setXAxisValue(assetClass.getAssetClassName());
				
			} else if(xAxis.getLabelType().equals("Sector")) {
	        	innerCube.setXAxisValue(sector.getSectorName());
	        	
	        } else if(xAxis.getLabelType().equals("Market Cap")){	        	
	        	if(denomination.equals("M")) {
	        		long marketCap = new Long(investment.getMarketCap()).longValue();
	        		double marketCapM = marketCap/1000000;
	        		innerCube.setXAxisValue(marketCapM + "M");
				} else if(denomination.equals("B")) {
					long marketCap = new Long(investment.getMarketCap()).longValue();
	        		double marketCapB = marketCap/1000000000.0d;
	        		innerCube.setXAxisValue(marketCapB + "B");					
				} 	        	
	        } else if(xAxis.getLabelType().equals("Volume")) {
	        	innerCube.setXAxisValue(new Long(investment.getVolume()).toString());
	        	
	        } else if(xAxis.getLabelType().equals("EPS")) {
	 	        	innerCube.setXAxisValue(new Float(investment.getEPS()).toString());
	 	        	
	        }  else if(xAxis.getLabelType().equals("P/E")) {
	        	innerCube.setXAxisValue(new Float(investment.getPE()).toString());
	        }
			
			// populate the Y axis values
			if(yAxis.getLabelType().equals("Asset Class")) {
				innerCube.setYAxisValue(assetClass.getAssetClassName());
				
			} else if(yAxis.getLabelType().equals("Sector")) {
	        	innerCube.setYAxisValue(sector.getSectorName());
	        	
	        } else if(yAxis.getLabelType().equals("Market Cap")){	        	
	        	if(denomination.equals("M")) {
	        		long marketCap = new Long(investment.getMarketCap()).longValue();
	        		double marketCapM = marketCap/1000000.0d;
	        		innerCube.setYAxisValue(marketCapM + "M");
				} else if(denomination.equals("B")) {
					long marketCap = new Long(investment.getMarketCap()).longValue();
					double marketCapB = marketCap/1000000000;
	        		innerCube.setYAxisValue(marketCapB + "B");					
				} 	        	
	        } else if(yAxis.getLabelType().equals("Volume")) {
	        	innerCube.setYAxisValue(new Long(investment.getVolume()).toString());
	        	
	        } else if(yAxis.getLabelType().equals("EPS")) {
	 	        	innerCube.setYAxisValue(new Float(investment.getEPS()).toString());
	 	        	
	        }  else if(yAxis.getLabelType().equals("P/E")) {
	        	innerCube.setYAxisValue(new Float(investment.getPE()).toString());
	        }
			
			
			// populate the Z axis values
			if(zAxis.getLabelType().equals("Asset Class")) {
				innerCube.setZAxisValue(assetClass.getAssetClassName());
				
			} else if(zAxis.getLabelType().equals("Sector")) {
	        	innerCube.setZAxisValue(sector.getSectorName());
	        	
	        } else if(zAxis.getLabelType().equals("Market Cap")){	        	
	        	if(denomination.equals("M")) {
	        		long marketCap = new Long(investment.getMarketCap()).longValue();
	        		double marketCapM = marketCap/1000000.0d;
	        		innerCube.setZAxisValue(marketCapM + "M");
				} else if(denomination.equals("B")) {
					long marketCap = new Long(investment.getMarketCap()).longValue();
					double marketCapB = marketCap/1000000000;
	        		innerCube.setZAxisValue(marketCapB + "B");					
				} 	        	
	        } else if(zAxis.getLabelType().equals("Volume")) {
	        	innerCube.setZAxisValue(new Long(investment.getVolume()).toString());
	        	
	        } else if(zAxis.getLabelType().equals("EPS")) {
	 	        	innerCube.setZAxisValue(new Float(investment.getEPS()).toString());
	 	        	
	        }  else if(zAxis.getLabelType().equals("P/E")) {
	        	innerCube.setZAxisValue(new Float(investment.getPE()).toString());
	        }
			
			
			innerCube.setSector(sector.getSectorName());
			innerCube.setSectorId(sector.getSectorId());
			innerCube.setAssetClassId(investment.getAssetClassId());
			
			innerCube.setColor(new float[] {0.0f, 1.0f, 0.0f}); //green
			
			
			// find all subsectors for the investment's parent sector
			ArrayList subSectorsList = null;
			try {
	        	subSectorsList = (ArrayList) new InvestmentHoldingDB(this.getServletContext()).	        	
	        		getSubSectorsForInvestment(new Integer(portfolioId), assetClass.getAssetClassId(), 
	        				investment.getSectorId());
	        } catch (Exception e) {
	        	String location = className + "doPost()";
	            this.logger.debugMessage(location);
	        }	
	        
	        subSectors = new String[subSectorsList.size()];
	        subSectorsList.toArray(subSectors);
			
			innerCube.setSubSectors(subSectors);
			
			companyInfo = new CompanyInfo();
			companyInfo.setName(investment.getCompanyName());
			companyInfo.setTicker(investment.getTickerSymbol());
			innerCube.setCompanyInfo(companyInfo);
			innerCubesTmpArrList.add(innerCube);
			
			System.out.println("innerCube.XAxisValue = " + innerCube.getXAxisValue());
				
			i++;
		}
		
		
		// ======================================================================
        // Figure out the axis labels for the Outer Cube
                
        CubeInfo cubeInfo = new CubeInfo();
        
        // Populate X axis labels depending on the values from db
        cubeInfo.setXAxisTitle(xAxis.getLabelType());
        if(xAxis.getLabelType().equals("Asset Class")) {
        	cubeInfo.setXAxisLabels(assetClassAxisLabels);
        } else if(xAxis.getLabelType().equals("Sector")) {
        	cubeInfo.setXAxisLabels(sectorAxisLabels);
        } else {
        	cubeInfo.setXAxisLabels(frequencyAxisLabels);
        }
        
        
        // Populate Y axis labels depending on the values from db
        cubeInfo.setYAxisTitle(yAxis.getLabelType());
        if(yAxis.getLabelType().equals("Asset Class")) {
        	cubeInfo.setYAxisLabels(assetClassAxisLabels);
        } else if(yAxis.getLabelType().equals("Sector")) {
        	cubeInfo.setYAxisLabels(sectorAxisLabels);
        } else {
        	cubeInfo.setYAxisLabels(frequencyAxisLabels);
        }
        
        
        // Populate Z axis labels depending on the values from db
        cubeInfo.setZAxisTitle(zAxis.getLabelType());
        if(zAxis.getLabelType().equals("Asset Class")) {
        	cubeInfo.setZAxisLabels(assetClassAxisLabels);
        } else if(zAxis.getLabelType().equals("Sector")) {
        	cubeInfo.setZAxisLabels(sectorAxisLabels);
        } else {
        	cubeInfo.setZAxisLabels(frequencyAxisLabels);
        }
        
        
        // ======================================================================        
        // Populate the outer cube with inner cubes
        innerCubes = new InnerCube[innerCubesTmpArrList.size()];
        i =0;
        for(Iterator<InnerCube> iter=innerCubesTmpArrList.iterator(); iter.hasNext();) {        	 
        	innerCubes[i] = iter.next();
        	i++;
        }        
        cubeInfo.setInnerCubes(innerCubes);
				
		
		// ======================================================================        
        // Prepare data to be sent back from the servlet to applet
		/*
		put watever objects you wants to return to applet.
		All objects must be serializable. */
		responseMap.put("LevelTicker", cubeInfo);
		
		
		out.writeObject(responseMap);
		out.flush();
		out.close();
		response.setStatus(HttpServletResponse.SC_NO_CONTENT);
		
	}

}
