package com.asc.servlets;

/**
 * Handles the login request sent by the client.
 * @created 29-Jan-2007 10:18:05 AM
 * @version 1.0
 */
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.net.*;
import java.util.*;
import javax.sql.*;
import java.sql.Connection;
import javax.naming.Context;
import javax.naming.InitialContext;
import java.math.*;

import com.core.server.common.*;
import com.core.server.database.*;
import com.core.server.loggers.*;
import com.core.server.objects.*;
import com.asc.server.objects.*;
import com.asc.server.database.*;


public class WorkerServlet  extends HttpServlet {

    private static String className = "WorkerServlet.";
    private AOGLogger logger = null;
    private ConfigFile cfg = null;
    private String url = null;


    public void init()throws ServletException {
        logger = (AOGLogger) getServletContext().getAttribute("logger");

        this.logger.debugMessage("Initializing WorkerServlet.......");
        this.cfg = (ConfigFile) getServletContext().getAttribute("configFile");
        this.url = (String) getServletContext().getAttribute("URL");
        this.logger.debugMessage("Initialization complete");
    }

    /**
     *
     * @param response
     * @param request
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response)  throws ServletException, IOException {

        doPost(request,response);

    }



    /**
     * Gateway function. On of the "process.." functions will be called depending on
     * the STATE value
     *
     * @param response
     * @param request
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String location = this.className+"doPost";
        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();

        try{
            // Initialize the this.logger
            this.logger.debugMessage(location);
            String state = new String(request.getParameter("STATE"));
            this.logger.debugMessage("STATE =  " + state);

            if (state.equals(StateValues.CREATE_USER)) {
                processCreateUser(request, response);
            } else if (state.equals(StateValues.CREATE_ROLE)) {
                processCreateRole(request, response);
            } else if (state.equals(StateValues.MODIFY_USER)) {
                processModifyUser(request, response);
            } else if (state.equals(StateValues.MODIFY_ROLE)) {
                processModifyRole(request, response);
            } else if (state.equals(StateValues.CREATE_PORTFOLIO)) {
                processCreatePortfolio(request, response);
            } else if (state.equals(StateValues.MODIFY_PORTFOLIO)) {
                processModifyPortfolio(request, response);
            } else if (state.equals(StateValues.CREATE_PORTFOLIO_AXIS)) {
                processCreateAxisPortfolio(request, response);
            } else if (state.equals(StateValues.MODIFY_PORTFOLIO_AXIS)) {
                processModifyAxisPortfolio(request, response);
            } else if (state.equals(StateValues.CREATE_INVESTMENT)) {
                processCreateInvestment(request, response);
            } else if (state.equals(StateValues.ADD_POSITION_BY_COMPANY)) {
                processAddInvestment(request, response);
            } else if (state.equals(StateValues.ADD_POSITION_BY_SYMBOL)) {
			    processAddInvestment(request, response);
            } else if (state.equals(StateValues.ADD_POSITION_BY_CUSIP)) {
			    processAddInvestment(request, response);
            } else if (state.equals(StateValues.SEARCH_USER)) {
			    processSearchUser(request, response);
            } else if (state.equals(StateValues.SEARCH_USER_RESULTS)) {
                processSearchUser(request, response);
            } else if (state.equals(StateValues.SEARCH_POSITION_BY_COMPANY)) {
                processAddInvestment(request, response);
		    } else if (state.equals(StateValues.REFRESH_USER)) {
                processRefreshUser(request, response);
            } else if (state.equals(StateValues.REFRESH_PORTFOLIO_AXIS) ||
                       state.equals(StateValues.REFRESH_PORTFOLIO_AXIS_X) ||
                       state.equals(StateValues.REFRESH_PORTFOLIO_AXIS_Y) ||
                       state.equals(StateValues.REFRESH_PORTFOLIO_AXIS_Z) ) {
                processRefreshPortfolioAxis(request, response);
            }
        } catch (Exception e) {
            logger.exceptionMessage(e);
            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }
        }

    }

    public void processCreateAxisPortfolio(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processCreateAxisPortfolio";
        PortfolioAxesDefinition newPortfolioAxesDefinition = null;
        PortfolioAxesDefinitionDB portfolioAxesDefinitionDAO = null;

        Axis xAxis = null;
        Axis yAxis = null;
        Axis zAxis = null;

        AxisDB axisDAO = null;

        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();

        int id;
        try{

           axisDAO = new AxisDB(context);
           portfolioAxesDefinitionDAO = new PortfolioAxesDefinitionDB(context);


           // Take the values in the X-axis section of the form
            xAxis = new Axis(request,"x");
             // Take the values in the Y-axis section of the form
            yAxis = new Axis(request,"y");

            // Take the values in the Z-axis section of the form
            zAxis = new Axis(request,"z");

            this.logger.debugMessage("New xAxis String: " + xAxis.toString());
            this.logger.debugMessage("New yAxis String: " + yAxis.toString());
            this.logger.debugMessage("New zAxis String: " + zAxis.toString());

            id = axisDAO.getId("X_Axis","Axis_ID");
            id++;

            xAxis.setAxisId(id);

			 // Create an entry in the database (x-axis)
			 axisDAO.createAxis(xAxis);


            id = axisDAO.getId("Y_Axis","Axis_ID");
            id++;
			 yAxis.setAxisId(id);

			 // Create an entry in the database (y-axis)
			 axisDAO.createAxis(yAxis);

            id = axisDAO.getId("Z_Axis","Axis_ID");
            id++;
            zAxis.setAxisId(id);

			 // Create an entry in the database (z-axis)
			 axisDAO.createAxis(zAxis);


            // Using the x-axis, y-axis, z-axis, portfolio_id, and user_id to make an entry in the
            // portfolio_axes_definition table
            newPortfolioAxesDefinition = new PortfolioAxesDefinition();
			newPortfolioAxesDefinition.setXAxisId(xAxis.getAxisId());
			newPortfolioAxesDefinition.setYAxisId(yAxis.getAxisId());
			newPortfolioAxesDefinition.setZAxisId(zAxis.getAxisId());
			newPortfolioAxesDefinition.setUserId(zAxis.getUserId());
			newPortfolioAxesDefinition.setPortfolioId(zAxis.getPortfolioId());


            portfolioAxesDefinitionDAO.createPortfolioAxesDefinition(newPortfolioAxesDefinition);

            String returnURL = this.url
	                   + "/portfolio_axes.jsp?MESSAGE=Portfolio Axes created successfully&STATE="
	                   + StateValues.MODIFY_PORTFOLIO_AXIS
	                   + "&id="
	                   +  xAxis.getPortfolioIdString();
  	     this.logger.debugMessage("return URL " + returnURL);
	     response.sendRedirect(returnURL);

        } catch (Exception e) {


            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }

        }


    }

    public void processModifyAxisPortfolio(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processModifyAxisPortfolio";

        Axis xAxis = null;
        Axis yAxis = null;
        Axis zAxis = null;

        AxisDB axisDAO = null;

        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();

        int id;
        try{

           axisDAO = new AxisDB(context);


            // Take the values in the X-axis section of the form
            xAxis = new Axis(request,"x");
             // Take the values in the Y-axis section of the form
            yAxis = new Axis(request,"y");
            // Take the values in the Z-axis section of the form
            zAxis = new Axis(request,"z");

            this.logger.debugMessage("New xAxis String: " + xAxis.toString());
            this.logger.debugMessage("New yAxis String: " + yAxis.toString());
            this.logger.debugMessage("New zAxis String: " + zAxis.toString());


			// Modify the existing entry in the database (x-axis)
			axisDAO.modifyAxis(xAxis);
			// Modify the existing  entry in the database (y-axis)
			axisDAO.modifyAxis(yAxis);
			// Modify the existing entry in the database (z-axis)
			axisDAO.modifyAxis(zAxis);


            String returnURL = this.url
	                   + "/portfolio_axes.jsp?MESSAGE=Portfolio Axes saved successfully&STATE="
	                   + StateValues.MODIFY_PORTFOLIO_AXIS
	                   + "&id="
	                   +  xAxis.getPortfolioIdString();
  	        this.logger.debugMessage("return URL " + returnURL);
	     response.sendRedirect(returnURL);

        } catch (Exception e) {


            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }

        }



    }

    public void processCreatePortfolio(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processCreatePortfolio";
        Portfolio newPortfolioEntry = null;
        PortfolioDB portfolioDAO = null;
         AxisDB axisDAO = null;
        PortfolioAxesDefinition newPortfolioAxesDefinition = null;
        PortfolioAxesDefinitionDB portfolioAxesDefinitionDAO = null;


        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();

        try{
            newPortfolioEntry = new Portfolio(request);
            portfolioDAO = new PortfolioDB(context);
            portfolioAxesDefinitionDAO = new PortfolioAxesDefinitionDB(context);

            int portfolio_id = portfolioDAO.getId("Portfolio","Portfolio_ID");
            portfolio_id++;

            newPortfolioEntry.setPortfolioId(portfolio_id);
            portfolio_id = portfolioDAO.createPortfolio(newPortfolioEntry);
            if (portfolio_id > 0 ) {
                    newPortfolioEntry.setPortfolioId(portfolio_id);
             }

            int x_id = portfolioDAO.getId("X_Axis","Axis_ID");
            int y_id = portfolioDAO.getId("Y_Axis","Axis_ID");
            int z_id = portfolioDAO.getId("Z_Axis","Axis_ID");

             // Create the initial X-Axis, Y-Axis, Z-Axis
			Axis xAxis = new Axis(new String("x"));
			xAxis.setPortfolioId(newPortfolioEntry.getPortfolioId());
			xAxis.setUserId(newPortfolioEntry.getUserId());
			xAxis.setScaleMin(new Long("0").longValue());
			xAxis.setScaleMax(new Long("0").longValue());
			x_id++;
			xAxis.setAxisId(x_id);
logger.debugMessage("X Axis: " + xAxis.toString());

			Axis yAxis = new Axis(new String("y"));
			yAxis.setPortfolioId(newPortfolioEntry.getPortfolioId());
			yAxis.setUserId(newPortfolioEntry.getUserId());
			yAxis.setScaleMin(new Long("0").longValue());
			yAxis.setScaleMax(new Long("0").longValue());
			y_id++;
			yAxis.setAxisId(y_id);
logger.debugMessage("Y Axis: " + yAxis.toString());

			Axis zAxis = new Axis(new String("z"));
			zAxis.setPortfolioId(newPortfolioEntry.getPortfolioId());
			zAxis.setUserId(newPortfolioEntry.getUserId());
			zAxis.setScaleMin(new Long("0").longValue());
			zAxis.setScaleMax(new Long("0").longValue());
			z_id++;
			zAxis.setAxisId(z_id);
logger.debugMessage("Z Axis: " + zAxis.toString());


            axisDAO = new AxisDB(context);

            // Modify the existing entry in the database (x-axis)
			axisDAO.createAxis(xAxis);
			// Modify the existing  entry in the database (y-axis)
			axisDAO.createAxis(yAxis);
			// Modify the existing entry in the database (z-axis)
			axisDAO.createAxis(zAxis);


            newPortfolioAxesDefinition = new PortfolioAxesDefinition();
			newPortfolioAxesDefinition.setXAxisId(xAxis.getAxisId());
			newPortfolioAxesDefinition.setYAxisId(yAxis.getAxisId());
			newPortfolioAxesDefinition.setZAxisId(zAxis.getAxisId());
			newPortfolioAxesDefinition.setUserId(newPortfolioEntry.getUserId());
			newPortfolioAxesDefinition.setPortfolioId(newPortfolioEntry.getPortfolioId());


            portfolioAxesDefinitionDAO.createPortfolioAxesDefinition(newPortfolioAxesDefinition);


             this.logger.debugMessage("New Portfolio String: " + newPortfolioEntry.toString());


             String returnURL = this.url
	                   + "/portfolio.jsp?MESSAGE=Portfolio created successfully&STATE="
	                   + StateValues.MODIFY_PORTFOLIO
	                   + "&id="
	                   +  newPortfolioEntry.getPortfolioIdString();
  	     this.logger.debugMessage("return URL " + returnURL);
	     response.sendRedirect(returnURL);

        } catch (Exception e) {


            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }

        }

}
    public void processModifyPortfolio(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processModifyPortfolio";
        Portfolio newPortfolioEntry = null;
        PortfolioDB portfolioDAO = null;
        InvestmentHoldingDB investmentHoldingDAO = null;


        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();
        int count = 0;
        int number_of_positions = 0;
        String investment_id = null;

        try{
            newPortfolioEntry = new Portfolio(request);
            portfolioDAO = new PortfolioDB(context);
            investmentHoldingDAO = new InvestmentHoldingDB(context);

            int portfolio_id = new Integer( (String) request.getParameter("id") ).intValue();

            newPortfolioEntry.setPortfolioId(portfolio_id);
            portfolioDAO.modifyPortfolio(newPortfolioEntry);

            this.logger.debugMessage("New Portfolio String: " + newPortfolioEntry.toString());



            number_of_positions = investmentHoldingDAO.getMaximumInvestmentId(portfolio_id);

            // update the number of positions for each investment_id
            // Loop through to get all of the positions held

			while (number_of_positions >= count) {

			   investment_id = new String("id_"+new Integer(count).toString());
			   this.logger.debugMessage("investment id: " + investment_id);
			   if (request.getParameter(investment_id) != null)
			   {
			        // Loop through and make all of the boradcasts Inactive that are checked
			        investmentHoldingDAO.updateNumberOfHoldings(request.getParameter(investment_id),count);
			   } // end of if statement
			   count++;
			} // end of while loop

            String returnURL = this.url
			                       + "/portfolio.jsp?MESSAGE=Portfolio information saved successfully&STATE="
			                       + StateValues.MODIFY_PORTFOLIO
			                       + "&id="
			                       + newPortfolioEntry.getPortfolioIdString();
			this.logger.debugMessage("return URL " + returnURL);
			response.sendRedirect(returnURL);

        } catch (Exception e) {
            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }
        }

    }

    public void processAddInvestment(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processAddInvestment";
        Investment newInvestmentEntry = null;
        InvestmentDB investmentDAO = null;

        InvestmentHolding newInvestmentHoldingEntry = null;
        InvestmentHoldingDB investmentHoldingDAO = null;

        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();
        String value = null;
        String state = new String(request.getParameter("STATE"));
        int portfolio_id = new Integer( (String) request.getParameter("id") ).intValue();
        int user_id = new Integer( (String) request.getParameter("user_id") ).intValue();

        try{
            investmentDAO = new InvestmentDB(context);
            investmentHoldingDAO = new InvestmentHoldingDB(context);

            // Read in the value entered by the user


            // Search for the investment in the
            if (state.equals(StateValues.ADD_POSITION_BY_CUSIP)) {

               value = new String(request.getParameter("cusip"));
               newInvestmentEntry = new Investment(investmentDAO.getInvestment("CUSIP",value));
               this.logger.debugMessage("newInvestmentEntry " + newInvestmentEntry.toString());
               if (newInvestmentEntry.getInvestmentId() == 0)
               {
					 String returnURL = this.url
						   + "/portfolio.jsp?MESSAGE=Investment Does not exist in the system&STATE="
						   + StateValues.MODIFY_PORTFOLIO
						   + "&id="
						   +  Integer.toString(portfolio_id);
  			     this.logger.debugMessage("return URL " + returnURL);
			     response.sendRedirect(returnURL);

			   }
            } else if (state.equals(StateValues.SEARCH_POSITION_BY_COMPANY)) {

               value = new String(request.getParameter("company_name"));

               // redirect to the users_list.jsp
 	           String returnURL = this.url
			                       + "/portfolio.jsp?STATE="
			                       + StateValues.SEARCH_COMPANY_RESULTS
			                       + "&company_name="+value
			                       + "&id="+portfolio_id;
   	           this.logger.debugMessage("return URL " + returnURL);
		       response.sendRedirect(returnURL);

            } else if (state.equals(StateValues.ADD_POSITION_BY_SYMBOL)) {

               value = new String(request.getParameter("ticker_symbol"));
               newInvestmentEntry = new Investment(investmentDAO.getInvestment("Ticker_Symbol",value));
               this.logger.debugMessage("newInvestmentEntry " + newInvestmentEntry.toString());
               if (newInvestmentEntry.getInvestmentId() == 0)
               {
					 String returnURL = this.url
						   + "/portfolio.jsp?MESSAGE=Investment Does not exist in the system&STATE="
						   + StateValues.MODIFY_PORTFOLIO
						   + "&id="
						   +  Integer.toString(portfolio_id);
  			     this.logger.debugMessage("return URL " + returnURL);
			     response.sendRedirect(returnURL);

			   }

            } else if (state.equals(StateValues.ADD_POSITION_BY_COMPANY )) {
               value = new String(request.getParameter("investment_id"));
               this.logger.debugMessage("value " + value);
               newInvestmentEntry = new Investment(investmentDAO.getInvestment("Investment_ID",value));
            }

            newInvestmentHoldingEntry = new InvestmentHolding(newInvestmentEntry.getInvestmentId(), portfolio_id, user_id);
            this.logger.debugMessage("newInvestmentHoldingEntry " + newInvestmentHoldingEntry.toString());

            // Check to see if the investment already exists
            if (!investmentHoldingDAO.investmentHoldingExists(newInvestmentHoldingEntry))
            {


				// Insert the position/investment in the porfolio
				investmentHoldingDAO.createInvestmentHolding(newInvestmentHoldingEntry);

				 String returnURL = this.url
						   + "/portfolio.jsp?MESSAGE=Position successfully added to Portfolio&STATE="
						   + StateValues.MODIFY_PORTFOLIO
						   + "&id="
						   +  Integer.toString(portfolio_id);
  			     this.logger.debugMessage("return URL " + returnURL);
			     response.sendRedirect(returnURL);
		    }
		    else
		    {
				 String returnURL = this.url
						   + "/portfolio.jsp?MESSAGE=Position already exists in Portfolio&STATE="
						   + StateValues.MODIFY_PORTFOLIO
						   + "&id="
						   +  Integer.toString(portfolio_id);
  			     this.logger.debugMessage("return URL " + returnURL);
			     response.sendRedirect(returnURL);

		    }

        } catch (Exception e) {


            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }

        }

    }




    public void processCreateInvestment(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processCreatePortfolio";
        Investment newInvestmentEntry = null;
        InvestmentDB investmentDAO = null;
        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();

        try{
            newInvestmentEntry = new Investment(request);
            investmentDAO = new InvestmentDB(context);

            int investment_id = investmentDAO.getId("Investments","investment_id");
            investment_id++;

            newInvestmentEntry.setInvestmentId(investment_id);
            investment_id = investmentDAO.createInvestment(newInvestmentEntry);
            if (investment_id > 0 ) {
                    newInvestmentEntry.setInvestmentId(investment_id);
             }


             this.logger.debugMessage("New Investment String: " + newInvestmentEntry.toString());


             String returnURL = this.url
	                   + "/investment.jsp?MESSAGE=Investment created successfully&STATE="
	                   + StateValues.CREATE_PORTFOLIO;
  	     this.logger.debugMessage("return URL " + returnURL);
	     response.sendRedirect(returnURL);

        } catch (Exception e) {


            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }

        }

    }


    public void processSearchUser(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processSearchUser";
        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();
        User newUserEntry = null;
        UserDB userDAO = null;


        String firstname = null;
        String lastname = null;
        String email = null;
        boolean searchByFirstName = false;
        boolean searchByLastName = false;
        boolean searchByEmail = false;

        try{
            newUserEntry = new User(request);
            userDAO = new UserDB(context);

            if (request.getParameter("searchFirstName") != null) {
                firstname = new String(request.getParameter("searchFirstName"));
                searchByFirstName = true;
            }
            if (request.getParameter("searchLastName") != null) {
                lastname = new String(request.getParameter("searchLastName"));
                searchByLastName = true;
            }
            if (request.getParameter("searchEmail") != null) {
                email = new String(request.getParameter("searchEmail"));
                searchByEmail = true;
            }
            // look


           // redirect to the users_list.jsp
 	       String returnURL = this.url
			                       + "/users_list.jsp?STATE="
			                       + StateValues.SEARCH_USER_RESULTS
			                       + "&searchFirstName="+firstname
			                       + "&searchLastName="+lastname
			                       + "&searchEmail="+email;
			                       ;
   	        this.logger.debugMessage("return URL " + returnURL);
		    response.sendRedirect(returnURL);


        } catch (Exception e) {


            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }

        }


    }





    public void processCreateUser(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processCreateUser";
        User newUserEntry = null;
        UserDB userDAO = null;
        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();

        try{
            newUserEntry = new User(request);
            userDAO = new UserDB(context);

            int user_id = userDAO.getId("Users","user_id");
            user_id++;

            // make sure the username doesn't already exist
            if (!userDAO.userNameExists(newUserEntry.getUserName())) {
                newUserEntry.setUserId(user_id);
                user_id = userDAO.createUser(newUserEntry);
                if (user_id > 0 ) {
                    newUserEntry.setUserId(user_id);
                }


                this.logger.debugMessage("New User String: " + newUserEntry.toString());

		    }
		    else {
			    this.logger.debugMessage("Login failed, returning to login page");
			    // redirect the user back to the login screen
			    // We must first set the state to a failed login
			    String returnURL = this.url
			                       + "/user.jsp?STATE="
			                       + StateValues.USERNAME_EXISTS;
			    this.logger.debugMessage("return URL " + returnURL);
			    response.sendRedirect(returnURL);

            }

            String returnURL = this.url
			                   + "/user.jsp?MESSAGE=User created successfully&STATE="
			                   + StateValues.CREATE_USER;
			this.logger.debugMessage("return URL " + returnURL);
			response.sendRedirect(returnURL);

        } catch (Exception e) {


            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }

        }

    }

    public void processCreateRole(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processCreateRole";
        Role newRoleEntry = null;
        RoleDB roleDAO = null;
        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();

        try{
            newRoleEntry = new Role(request);
            roleDAO = new RoleDB(context);

            int role_id = roleDAO.getId("Roles","role_id");
            role_id++;
            // make sure the rolename doesn't already exist
            if (!roleDAO.roleNameExists(newRoleEntry)) {
                newRoleEntry.setRoleId(role_id);
                role_id = roleDAO.createRole(newRoleEntry);
                if (role_id > 0 ) {
                    newRoleEntry.setRoleId(role_id);
                }


                this.logger.debugMessage("New Role String: " + newRoleEntry.toString());

		    }
		    else {
			    this.logger.debugMessage("Login failed, returning to login page");
			    // redirect the role back to the login screen
			    // We must first set the state to a failed login
			    String returnURL = this.url
			                       + "/role.jsp?STATE="
			                       + StateValues.USERNAME_EXISTS;
			    this.logger.debugMessage("return URL " + returnURL);
			    response.sendRedirect(returnURL);

            }

            String returnURL = this.url
			                       + "/role.jsp?MESSAGE=Role created successfully&STATE="
			                       + StateValues.CREATE_ROLE;
			this.logger.debugMessage("return URL " + returnURL);
			response.sendRedirect(returnURL);

        } catch (Exception e) {
            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }
        }

    }




    public void processModifyUser(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processModifyUser";
        User newUserEntry = null;
        UserDB userDAO = null;
        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();

        try{
            newUserEntry = new User(request);
            userDAO = new UserDB(context);

            int user_id = new Integer( (String) request.getParameter("id") ).intValue();

            newUserEntry.setUserId(user_id);
            userDAO.modifyUser(newUserEntry);

            this.logger.debugMessage("Modified User String: " + newUserEntry.toString());


            String returnURL = this.url
			                       + "/user.jsp?MESSAGE=User information saved successfully&STATE="
			                       + StateValues.MODIFY_USER
			                       + "&id="
			                       + newUserEntry.getUserIdString();
			this.logger.debugMessage("return URL " + returnURL);
			response.sendRedirect(returnURL);

        } catch (Exception e) {
            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }
        }

    }


    public void processModifyRole(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processModifyRole";
        Role newRoleEntry = null;
        RoleDB roleDAO = null;
        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();
        this.logger.debugMessage(location);

        try{
            newRoleEntry = new Role(request);
            roleDAO = new RoleDB(context);

            int role_id = new Integer( (String) request.getParameter("id") ).intValue();

            newRoleEntry.setRoleId(role_id);

            String role_type_id = null;
            if (request.getParameter("role_type_id") != null) {
                role_type_id = new String(request.getParameter("role_type_id"));
                newRoleEntry.setRoleTypeId(new Integer(role_type_id).intValue());
            }
            this.logger.debugMessage("role_type_id: " + role_type_id);

            String name = null;
            if (request.getParameter("name") != null) {
                name = new String(request.getParameter("name"));
                newRoleEntry.setName(name);
            }

            String description = null;
            if (request.getParameter("description") != null) {
                description = new String(request.getParameter("description"));
                newRoleEntry.setDescription(description);
            }

            roleDAO.modifyRole(newRoleEntry);

            this.logger.debugMessage("New Role String: " + newRoleEntry.toString());


            String returnURL = this.url
			                       + "/role.jsp?MESSAGE=Role information saved successfully&STATE="
			                       + StateValues.MODIFY_ROLE
			                       + "&id="
			                       + newRoleEntry.getRoleIdString();
			this.logger.debugMessage("return URL " + returnURL);
			response.sendRedirect(returnURL);

        } catch (Exception e) {
            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }
        }

    }



    public void processRefreshUser(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processRefreshUser";
        User newUserEntry = null;
        UserDB userDAO = null;
        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();
        String parent_user_id = null;
        String state = null;
        String returnURL = null;
        String id = null;

        try{
            newUserEntry = new User(request);

            this.logger.debugMessage("Refresh User String: " + newUserEntry.toString());
            id = new String(request.getParameter("id"));

            String role_id_string = null;
            this.logger.debugMessage("Refresh User String: " + request.getParameter("id"));

            if (id.length() != 0) {
				 returnURL = this.url
									   + "/user.jsp?STATE="
									   + StateValues.MODIFY_USER
									   + "&id="
									   + id
									   + "&parent_role_type="
									   + newUserEntry.getRoleIdString();

            }
            else
            {

				returnURL = this.url
									   + "/user.jsp?STATE="
									   + StateValues.CREATE_USER
									   + "&parent_role_type="
									   + newUserEntry.getRoleIdString();

            }
			this.logger.debugMessage("return URL " + returnURL);
			response.sendRedirect(returnURL);



        } catch (Exception e) {
            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }
        }

    }




    public void processRefreshPortfolioAxis(HttpServletRequest request, HttpServletResponse response){
        String location = this.className+"processRefreshPortfolioAxis";
            String state = new String(request.getParameter("STATE"));
            this.logger.debugMessage("STATE =  " + state);


        Axis xAxis = null;
        Axis yAxis = null;
        Axis zAxis = null;

        AxisDB axisDAO = null;

        ServletConfig config = getServletConfig();
        ServletContext context = config.getServletContext();
        String x_axis_label_type = null;
        String y_axis_label_type = null;
        String z_axis_label_type = null;

        String id = null;
//        BigInteger insn = new BigInteger("300000000000");

        try{

            if (request.getParameter("x_axis_label_type") != null)
            {
   			   x_axis_label_type = new String(request.getParameter("x_axis_label_type"));
			}
			else
			{
				x_axis_label_type = new String("Market Cap");
			}
            if (request.getParameter("y_axis_label_type") != null)
            {
   			   y_axis_label_type = new String(request.getParameter("y_axis_label_type"));
			}
			else
			{
				y_axis_label_type = new String("Market Cap");
			}
            if (request.getParameter("z_axis_label_type") != null)
            {
   			   z_axis_label_type = new String(request.getParameter("z_axis_label_type"));
			}
			else
			{
				z_axis_label_type = new String("Market Cap");
			}

            axisDAO = new AxisDB(context);

            // Take the values in the X-axis section of the form
            xAxis = new Axis(request,"x");
             // Take the values in the Y-axis section of the form
            yAxis = new Axis(request,"y");
            // Take the values in the Z-axis section of the form
            zAxis = new Axis(request,"z");


            // If we are refreshing the label then we must refresh the value
            if (state.equals(StateValues.REFRESH_PORTFOLIO_AXIS_X))
            {
              xAxis.setLabelName(new String(""));
			}
			else if (state.equals(StateValues.REFRESH_PORTFOLIO_AXIS_Y))
            {
              yAxis.setLabelName(new String(""));
			}
			else if (state.equals(StateValues.REFRESH_PORTFOLIO_AXIS_Z))
            {
              zAxis.setLabelName(new String(""));

			}


            //Depending on the state value we need to set some default values
            if (state.equals(StateValues.REFRESH_PORTFOLIO_AXIS_X))
            {
			   if (x_axis_label_type.equals("Market Cap"))
			   {
                   xAxis.setScaleMin(new Long("5000000").longValue());
                   xAxis.setScaleMax(new Long("500000000000").longValue());
  //                 xAxis.setScaleMax(5000000);
		       }
			   else if (x_axis_label_type.equals("Volume"))
			   {
                   xAxis.setScaleMin(new Long("1000").longValue());
                   xAxis.setScaleMax(new Long("100000000").longValue());
		       }
			   else if (x_axis_label_type.equals("P/E"))
			   {
                   xAxis.setScaleMin(new Long("0").longValue());
                   xAxis.setScaleMax(new Long("30").longValue());
		       }
			   else if (x_axis_label_type.equals("EPS"))
			   {
                   xAxis.setScaleMin(new Long("0").longValue());
                   xAxis.setScaleMax(new Long("5").longValue());
		       }
            }

            if (state.equals(StateValues.REFRESH_PORTFOLIO_AXIS_Y))
            {
			   if (y_axis_label_type.equals("Market Cap"))
			   {
                   yAxis.setScaleMin(new Long("5000000").longValue());
                   yAxis.setScaleMax(new Long("500000000000").longValue());

	           }
			   else if (y_axis_label_type.equals("Volume"))
			   {
                   yAxis.setScaleMin(new Long("1000").longValue());
                   yAxis.setScaleMax(new Long("100000000").longValue());
		       }
			   else if (y_axis_label_type.equals("P/E"))
			   {
                   yAxis.setScaleMin(new Long("0").longValue());
                   yAxis.setScaleMax(new Long("30").longValue());
		       }
			   else if (y_axis_label_type.equals("EPS"))
			   {
                   yAxis.setScaleMin(new Long("0").longValue());
                   yAxis.setScaleMax(new Long("5").longValue());
		       }
            }


            if (state.equals(StateValues.REFRESH_PORTFOLIO_AXIS_Z))
            {
			   if (z_axis_label_type.equals("Market Cap"))
			   {
                   zAxis.setScaleMin(new Long("5000000").longValue());
                   zAxis.setScaleMax(new Long("500000000000").longValue());

  //                 xAxis.setScaleMax(5000000);
		       }
			   else if (z_axis_label_type.equals("Volume"))
			   {
                   zAxis.setScaleMin(new Long("1000").longValue());
                   zAxis.setScaleMax(new Long("100000000").longValue());
		       }
			   else if (z_axis_label_type.equals("P/E"))
			   {
                   zAxis.setScaleMin(new Long("0").longValue());
                   zAxis.setScaleMax(new Long("30").longValue());
		       }
			   else if (z_axis_label_type.equals("EPS"))
			   {
                   zAxis.setScaleMin(new Long("0").longValue());
                   zAxis.setScaleMax(new Long("5").longValue());
		       }
            }


            this.logger.debugMessage("New xAxis String: " + xAxis.toString());
            this.logger.debugMessage("New yAxis String: " + yAxis.toString());
            this.logger.debugMessage("New zAxis String: " + zAxis.toString());


			// Modify the existing entry in the database (x-axis)
			axisDAO.modifyAxis(xAxis);
			// Modify the existing  entry in the database (y-axis)
			axisDAO.modifyAxis(yAxis);
			// Modify the existing entry in the database (z-axis)
			axisDAO.modifyAxis(zAxis);


            String returnURL = this.url
	                   + "/portfolio_axes.jsp?&STATE="
	                   + StateValues.MODIFY_PORTFOLIO_AXIS
	                   + "&id="
	                   +  xAxis.getPortfolioIdString()
	                   + "&x_axis_label_type="
	                   +  x_axis_label_type
	                   + "&y_axis_label_type="
	                   +  y_axis_label_type
	                   + "&z_axis_label_type="
	                   +  z_axis_label_type;

  	        this.logger.debugMessage("return URL " + returnURL);
	        response.sendRedirect(returnURL);

        } catch (Exception e) {


            try{
                WebPageException.createExceptionPage(this.cfg, response, this.logger, e, this.url);
                new ExceptionDB(getServletContext()).writeExceptionRecord(e);
                WebPageException.sendExceptionEmail(request, response, e, this.cfg, this.logger, getServletContext());
            } catch (Exception webE) {
                webE.printStackTrace();
            }

        }
    }

    /**
     *
     * @exception Throwable
     */
    public void finalize()
    throws Throwable{

    }





}