package com.asc.utils;

import javax.media.j3d.Appearance;
import javax.media.j3d.ColoringAttributes;
import javax.media.j3d.LineAttributes;
import javax.media.j3d.Material;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.TransparencyAttributes;
import javax.vecmath.Color3f;

public class CubeAppearance extends Appearance implements 
	PortfolioCubeConstants {
	
	
	/*****************************************************************
	 * Returns the Wireframe Appearance
 	 *****************************************************************/
    public Appearance getWireframeAppearance(Color3f color) {
        Appearance app = new Appearance();
        //Color awtColor = new Color(255, 255, 0);// use any Color you like
        //Color3f color = new Color3f(awtColor);
        ColoringAttributes ca = new ColoringAttributes();
        ca.setColor(color);
        app.setColoringAttributes(ca);
        PolygonAttributes pa = new PolygonAttributes();
        pa.setPolygonMode(pa.POLYGON_LINE);
        pa.setCullFace(pa.CULL_NONE);
        pa.setPolygonOffset(-1000.0f);
        app.setPolygonAttributes(pa);
        return app;
    }


    /*****************************************************************
	 * Returns the Yellow Appearance
 	 *****************************************************************/
    public Appearance createYellowAppearance() {

    	  //Define colors
		  //These are RGB values, eg 1.0f, 0.0f, 0.0f is bright red
		  Color3f black         = new Color3f(0.9f,  0.9f,  0.2f);
		  Color3f yellow        = new Color3f(0.5f,  0.9f,  0.2f);
		  Color3f ambientYellow = new Color3f(0.5f,  0.6f,  0.2f);
		  Color3f specular      = new Color3f(1.0f,  0.0f,  0.0f);

		  TransparencyAttributes ta = new TransparencyAttributes();
		  ta.setTransparencyMode (ta.NICEST);
		  ta.setTransparency (0.0f);

		  //The Material class defines the appearance of an object under illumination.
		  //Fields in the constructor are ambientColor, emissiveColor, diffuseColor, specularColor and shininess
		  Material yellowMaterial = new Material(ambientYellow, ambientYellow, yellow, specular, 90.0f);
		  //Material also enables or disables lighting
		  yellowMaterial.setLightingEnable(true);


		  Appearance yellowAppearance = new Appearance();
		  ColoringAttributes ca = new ColoringAttributes();
		  ca.setColor( new Color3f(0.50f, 0.50f, 0.25f) );
		  ca.setShadeModel(ColoringAttributes.NICEST);
		  yellowAppearance.setColoringAttributes(ca);
		  yellowAppearance.setTransparencyAttributes(ta);
		  yellowAppearance.setMaterial(yellowMaterial);

		  //PointAttributes pa = new PointAttributes(3, true);
		  //yellowAppearance.setPointAttributes(pa);

		  LineAttributes la = new LineAttributes(5, LineAttributes.PATTERN_SOLID, true);
		  yellowAppearance.setLineAttributes(la);

		  return yellowAppearance;

    }
    
    
    /*****************************************************************
	 * Returns the Text Appearance
 	 *****************************************************************/
    public Appearance createTextAppearance() {

    	 //Define colors
		  Color3f objColor      = new Color3f(0.6f,  0.6f,  0.6f);
		  Color3f eColor        = new Color3f(0.0f,  0.0f,  0.0f);
		  Color3f sColor	 	= new Color3f(1.0f,  1.0f,  1.0f);
		  		 

		  //The Material class defines the appearance of an object under illumination.
		  //Fields in the constructor are ambientColor, emissiveColor, diffuseColor, specularColor and shininess
		  Material m = new Material(objColor, eColor, objColor, sColor, 100.0f);
		  Appearance a = new Appearance();
		  m.setLightingEnable(true);
		  
		  a.setMaterial(m);
		  
		  return a;

    }



    /*****************************************************************
	 * Returns the Transparent Appearance
 	 *****************************************************************/
    public Appearance createTransparentAppearance() {

   	    //Define colors
	    //These are RGB values, eg 1.0f, 0.0f, 0.0f is bright red
    	// PortfolioCubeConstants.lightGrey;
    	Color3f ambient 	  = PortfolioCubeConstants.blue1;
    	Color3f diffuse 	  = PortfolioCubeConstants.blue1;   	
		Color3f emmisive      = PortfolioCubeConstants.blue1;		  
		Color3f specular      = PortfolioCubeConstants.blue1;
		

		TransparencyAttributes ta = new TransparencyAttributes();
		ta.setTransparencyMode (TransparencyAttributes.NICEST);
		ta.setTransparency (0.5f);


		//The Material class defines the appearance of an object under illumination.
		//Fields in the constructor are ambientColor, emissiveColor, diffuseColor, specularColor and shininess
		Material m = new Material(ambient, emmisive, diffuse, specular, 90.0f);		  
		m.setLightingEnable(true);

		  
		Appearance app = new Appearance();
		
		ColoringAttributes ca = new ColoringAttributes();
		//ca.setColor( new Color3f(0.50f, 0.50f, 0.25f) );
		ca.setColor( PortfolioCubeConstants.blue1 );
		ca.setShadeModel(ColoringAttributes.NICEST);
		  
		app.setColoringAttributes(ca);
		app.setTransparencyAttributes(ta);
		app.setMaterial(m);

		LineAttributes la = new LineAttributes(5, LineAttributes.PATTERN_SOLID, true);
		app.setLineAttributes(la);

		  
		return app;
		
    }

    

    /*****************************************************************
	 * Returns the Red Appearance
 	 *****************************************************************/
    public Appearance createRedAppearance() {


		  //The Material class defines the appearance of an object under illumination.
		  //Fields in the constructor are ambientColor, emissiveColor, diffuseColor, specularColor and shininess
		  Material redMaterial = new Material(red, red, red, red, 90.0f);
		  //Material also enables or disables lighting
		  redMaterial.setLightingEnable(true);


		  Appearance redAppearance = new Appearance();
		  redAppearance.setMaterial(redMaterial);

		  LineAttributes la = new LineAttributes(5, LineAttributes.PATTERN_SOLID, true);
		  redAppearance.setLineAttributes(la);

		  return redAppearance;

    }




    /*****************************************************************
	 * Returns the Green Appearance
 	 *****************************************************************/
    public Appearance createGreenAppearance() {


		  //The Material class defines the appearance of an object under illumination.
		  //Fields in the constructor are ambientColor, emissiveColor, diffuseColor, specularColor and shininess
		  Material greenMaterial = new Material(darkGreen, darkGreen, darkGreen, darkGreen, 90.0f);
		  //Material also enables or disables lighting
		  greenMaterial.setLightingEnable(true);


		  Appearance greenAppearance = new Appearance();
		  greenAppearance.setMaterial(greenMaterial);

		  LineAttributes la = new LineAttributes(5, LineAttributes.PATTERN_SOLID, true);
		  greenAppearance.setLineAttributes(la);

		  return greenAppearance;

    }
    

}
