/*************************************************************************
 * Author : Kavitha Subramani
 * Date   : Aug 28 2007
 * Version: 1.0
**************************************************************************/

package com.asc.utils;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import com.asc.utils.*;
import com.asc.utils.*;


public class FloatLabelJSliderHorizontal extends JPanel implements ChangeListener,
	PortfolioCubeConstants {

    public JSlider		slider;
    JLabel		valueLabel;
    Vector		listeners = new Vector();
    float		min, max, resolution, current, scale;
    int			minInt, maxInt, curInt;;
    int			intDigits, fractDigits;
    float		minResolution = 0.001f;

    // default slider with name, resolution = 0.1, min = 0.0, max = 1.0 inital
    // 0.5
    public FloatLabelJSliderHorizontal(String name) {
		this(name, 0.1f, 0.0f, 1.0f, 0.5f);
    }

    public FloatLabelJSliderHorizontal(String name, float resolution, float min, float max,
	float current) {

		this.resolution = resolution;
		this.min = min;
		this.max = max;
		this.current = current;

		if (resolution < minResolution) {
	    	resolution = minResolution;
		}

		// round scale to nearest integer fraction. i.e. 0.3 => 1/3 = 0.33
		scale = (float) Math.round(1.0f / resolution);
		resolution = 1.0f / scale;

		// get the integer versions of max, min, current
		minInt = Math.round(min * scale);
		maxInt = Math.round(max * scale);
		curInt = Math.round(current * scale);

   	 	// sliders use integers, so scale our floating point value by "scale"
    	// to make each slider "notch" be "resolution".  We will scale the
		// value down by "scale" when we get the event.
    	slider = new JSlider(JSlider.HORIZONTAL, minInt, maxInt, curInt);    	
    	slider.addChangeListener(this);

		valueLabel = new JLabel(" ");

		// set the initial value label
		setLabelString();

		// add min and max labels to the slider
		//Hashtable labelTable = new Hashtable();
		//labelTable.put(new Integer(minInt),
		//		new JLabel(nf.format(min)));
		//labelTable.put(new Integer(maxInt),
		//		new JLabel(nf.format(max)));
		//slider.setLabelTable(labelTable);
		//slider.setPaintLabels(true);


		/* layout to align left */
		setLayout(new BorderLayout());
		Box box = new Box(BoxLayout.X_AXIS);
		add(box, BorderLayout.CENTER);

		box.add(new JLabel(name));
		box.add(slider);		
		//box.add(valueLabel);
	}



    public void setMinorTickSpacing(float spacing) {
	int intSpacing = Math.round(spacing * scale);
	slider.setMinorTickSpacing(intSpacing);
    }

    public void setMajorTickSpacing(float spacing) {
	int intSpacing = Math.round(spacing * scale);
	slider.setMajorTickSpacing(intSpacing);
    }

    public void setPaintTicks(boolean paint) {
	slider.setPaintTicks(paint);
    }

    public void addFloatListener(FloatListener listener) {
	listeners.add(listener);
    }

    public void removeFloatListener(FloatListener listener) {
	listeners.remove(listener);
    }


    public void stateChanged(ChangeEvent e) {
        JSlider source = (JSlider)e.getSource();
	// get the event type, set the corresponding value.
	// Sliders use integers, handle floating point values by scaling the
	// values by "scale" to allow settings at "resolution" intervals.
	// Divide by "scale" to get back to the real value.
	curInt = source.getValue();
	current = curInt / scale;

	valueChanged();
    }

    public void setValue(float newValue) {
        boolean changed = (newValue != current);
	current = newValue;
	if (changed) {
            valueChanged();
	}
    }


    private void valueChanged() {
	// update the label
        setLabelString();

	// notify the listeners
	FloatEvent event = new FloatEvent(this, current);
	for (Enumeration e = listeners.elements() ; e.hasMoreElements() ;) {
	    FloatListener listener = (FloatListener) e.nextElement();
	    listener.floatChanged(event);
	}
    }

    void setLabelString() {
    	
    	// Need to muck around to try to make sure that the width of the label
    	// is wide enough for the largest value.  Pad the string
    	// be large enough to hold the largest value.
    	int pad = 5;  // fudge to make up for variable width fonts
    	float maxVal = Math.max(Math.abs(min), Math.abs(max));
    	intDigits = Math.round((float)(Math.log(maxVal) / Math.log(10))) + pad;
    	if (min < 0) {
    		intDigits++; // add one for the '-'
    	}
    	
    	// fractDigits is num digits of resolution for fraction. Use base 10 log
    	// of scale, rounded up, + 2.
    	fractDigits = (int) Math.ceil((Math.log(scale) / Math.log(10)));
    	nf.setMinimumFractionDigits(fractDigits);
    	nf.setMaximumFractionDigits(fractDigits);
    	String value = nf.format(current);
    	while (value.length() < (intDigits + fractDigits)) {
    		value = value + "  ";
    	}
        
    	valueLabel.setText(value);
    	
   }



}
